#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SIZE 100

// Štruktúra pre uloženie informácií o študentovi
struct student {
    char name[2 * SIZE]; // Zvýšená veľkosť pre celé meno (meno + priezvisko)
    int votes;
};

// Funkcia pre porovnanie dvoch študentov
int compare(const void* p1, const void* p2) {
    struct student* s1 = (struct student*)p1;
    struct student* s2 = (struct student*)p2;
    
    // Porovnáme najprv počet hlasov
    if (s1->votes != s2->votes) {
        return s2->votes - s1->votes; // Zoradíme zostupne podľa počtu hlasov
    } else {
        // Ak majú rovnaký počet hlasov, zoradíme lexikograficky podľa mena
        return strcmp(s1->name, s2->name);
    }
}

// Funkcia na vyhľadanie študenta v databáze
int find_student(struct student* students, int size, const char* name) {
    for (int i = 0; i < size; ++i) {
        if (strcmp(students[i].name, name) == 0) {
            return i; // Nájdený študent
        }
    }
    return -1; // Študent nebol nájdený
}

int main() {
    struct student database[SIZE]; // Databáza študentov
    memset(database, 0, SIZE * sizeof(struct student)); // Inicializácia pamäte
    
    int size = 0; // Aktuálny počet študentov v databáze
    
    // Načítanie hlasov zo vstupu
    char line[2 * SIZE]; // Zvýšená veľkosť pre celý riadok (meno + priezvisko)
    while (fgets(line, 2 * SIZE, stdin) != NULL) {
        char name[2 * SIZE]; // Zvýšená veľkosť pre celé meno (meno + priezvisko)
        int votes;
        // Rozdelenie riadku na počet hlasov a meno študenta
        if (sscanf(line, "%d %[^\n]", &votes, name) != 2) {
            fprintf(stderr, "Chybný formát vstupu!\n");
            break;
        }
        
        // Vyhľadanie študenta v databáze
        int index = find_student(database, size, name);
        if (index == -1) {
            // Nový študent
            if (size < SIZE) {
                // Pridanie študenta do databázy
                strcpy(database[size].name, name);
                database[size].votes = votes;
                size++;
            } else {
                fprintf(stderr, "Databáza je plná!\n");
                break;
            }
        } else {
            // Existujúci študent, pridáme hlasy
            database[index].votes += votes;
        }
    }
    
    // Zoradenie databázy podľa počtu hlasov a mena
    qsort(database, size, sizeof(struct student), compare);
    
    // Výpis výsledkov
    printf("Vysledky:\n");
    for (int i = 0; i < size; ++i) {
        printf("%d %s\n", database[i].votes, database[i].name);
    }
    return 0;
}
